% Script that carries out the fitting between the DIC results anf the
% theoretical model (Williams' Model)

close all

tic


% Adjusting the initial translation
sX = +0.5447;   % X translation
sY = -0.1173;   % Y translation
sZ = 0.0258;    % Z translation
xyzDICnew1(:,1) = xyzDIC(:,1) + sX;
xyzDICnew1(:,2) = xyzDIC(:,2) + sY;
xyzDICnew1(:,3) = xyzDIC(:,3) + sZ;

% Adjusting the initial orientation
alpha_r = -deg2rad(0);
npoints = size(xyzDIC,1);
eul = [alpha_r 9*10^(-4) 0];  
rotm = eul2rotm(eul);
xyzDICnew2 = rotm*xyzDICnew1';
xyzDICnew2 = xyzDICnew2';


xdata = xyzDICnew2(:,1:2);  % xdata for the fitting
ydata = xyzDICnew2(:,3);    % ydata for the fitting

% Downsample of the data for the fitting for the sake of speed
ds = 5; 
xdata = downsample(xdata,ds);
ydata = downsample(ydata,ds);

% Theoretical model
fun = @(par,xdata)WilliamModel(xdata, par);
% "par" is the vector of the parameters that are determined by the fitting. 
% The fitting is performetd at line 94 by the function "lsqcurvefit"

nmax = 15; % Number of terms of the William's expansion considered
           
% Initialization
KIserie = NaN(1,nmax);
KIIserie = NaN(1,nmax);
Tserie = NaN(1,nmax);
sXserie = NaN(1,nmax);
sYserie = NaN(1,nmax);
sZserie = NaN(1,nmax);
alpha_serie = NaN(1,nmax);

% STIFs initial values 
KI = 1400;               % [Mpa*mm^0.5]
T =  -22;                % [Mpa]
KII = 0;                 % [Mpa*mm^0.5]

nvet = 2:nmax;

% Loop that computes the fitting parameters for the William expansion, 
% increasing the mumber of terms from 2 to nmax
for n = 2:nmax % n is the number of terms of William's expansion
    tic
    L = 2*n + 4;
    x0 = zeros(1,L);  % Initial values initialization
    lb = zeros(1,L);  % Lower bound initialization
    ub = zeros(1,L);  % Upper bound initialization
    % x0, lb, ub are in the form: [a(i)...b(i)...deltaX,deltaY,deltaZ,alpha]; 
    % x0, lb, ub for the terms of type "a" -------------------------------%
    x0(1:2) = [KI/sqrt(2*pi) T/4];      
    lb(1:2) = [0    -300];                 
    ub(1:2) = [3000 +300];               
    for i = 3:(L-4)/2
        x0(i) = 0;        
        lb(i) = -100;             
        ub(i) = +100;       
    end
    % x0, lb, ub for the terms of type "b" -------------------------------%
    Lb = (L-4)/2;
    x0(Lb+1:Lb+2) = [-KII/sqrt(2*pi) 0];
    lb(Lb+1:Lb+2) = [-300 -300];
    ub(Lb+1:Lb+2) = [+300 +300];
    for i = (Lb+3:L-4)
        x0(i) = 0;
        lb(i) = -100;
        ub(i) = +100;
    end
    % x0, lb, ub for the terms of type [deltaX, deltaY, deltaZ, alpha]----%
    delta = 0.1;
    deltaZ = 0.5;
    x0(L-3:L) = [+0;       +0;      +0;       +0   ];  
    lb(L-3:L) = [-delta;   -delta;  -deltaZ;  -0.01];
    ub(L-3:L) = [+delta;   +delta;  +deltaZ;  +0.01];
    
    %options = optimset('MaxFunEvals',100000, 'MaxIter',100000, 'TolFun',1.0000e-11,'TolX',1.0000e-11);
    
    % Non-linear curve fitting
    [par_opt,resnorm] = lsqcurvefit(fun,x0,xdata,ydata,lb,ub);%,options);
    
    % Saving the values of the parameters on vectors
    KIserie(n) = par_opt(1)*sqrt(2*pi);
    KIIserie(n) = -par_opt((L-4)/2+1)*sqrt(2*pi);
    Tserie(n) = 4*par_opt(2);
    sXserie(n) = par_opt(L-3);
    sYserie(n) = par_opt(L-2);
    sZserie(n) = par_opt(L-1);
    alpha_serie(n) = par_opt(L);
    
    disp(n);
    toc
end

meanKI = (sum(KIserie(8:nmax))/(nmax-7))*10^(-3/2); % Mean of KI from the 8th term to the nmax term
meanKII = (sum(KIIserie(8:nmax))/(nmax-7))*10^(-3/2); % Mean of KII from the 8th term to the nmax term


%-------------------------------------------------------------------------%
%--------------- Various figures (results for different n)----------------%
%-------------------------------------------------------------------------%

% SIFs
figure 
% KI
plot(nvet,KIserie(2:nmax)*10^(-3/2),'b--o','Linewidth',2);
xlabel('n');
ylabel('SIFs [MPa*m^{0.5}]');
%title('K_{I}');
xlim([2 nmax])
ylim([-10 160]) %[-5 100]
xL = xlim;
% line(xL, [lb(1)*10^(-3/2)*sqrt(2*pi) lb(1)*10^(-3/2)*sqrt(2*pi),], 'Color','red','LineWidth',2);  
% line(xL, [ub(1)*10^(-3/2)*sqrt(2*pi) ub(1)*10^(-3/2)*sqrt(2*pi),], 'Color','red','LineWidth',2); 
line(xL, [meanKI meanKI], 'Color','cyan','LineWidth',2);
hold on
% KII
plot(nvet,KIIserie(2:nmax)*10^(-3/2),'g--o','Linewidth',2);
% line(xL, [-lb(Lb+1)*10^(-3/2)*sqrt(2*pi) -lb(Lb+1)*10^(-3/2)*sqrt(2*pi)], 'Color','red','LineWidth',2);  
% line(xL, [-ub(Lb+1)*10^(-3/2)*sqrt(2*pi) -ub(Lb+1)*10^(-3/2)*sqrt(2*pi)], 'Color','red','LineWidth',2);  
line(xL, [meanKII meanKII], 'Color','yellow','LineWidth',2);
legend('K_{I}','mean K_{I}','K_{II}','mean K_{II}')
set(gca,'fontsize',10);
set(gca,'FontWeight','bold');
set(gca,'LineWidth',1.5);
set(gca,'TickLength',[0.018, 0.018]);

% % T-Stress
% figure
% plot(nvet,Tserie(2:nmax),'g--o','Linewidth',2);
% xlabel('n');
% ylabel('T [MPa]');
% title('T [MPa]');
% xlim([2 nmax])
% 
% % deltaX
% figure
% plot(nvet,sXserie(2:nmax),'b--o','Linewidth',2);
% xlabel('n');
% ylabel('\DeltaX [mm]');
% title('\DeltaX [mm]');
% xlim([2 nmax])
% 
% %deltaY
% figure
% plot(nvet,sYserie(2:nmax),'b--o','Linewidth',2);
% xlabel('n');
% ylabel('\DeltaY [mm]');
% title('\DeltaY [mm]');
% xlim([2 nmax])
% 
% %deltaZ
% figure
% plot(nvet,sZserie(2:nmax),'b--o','Linewidth',2);
% xlabel('n');
% ylabel('\DeltaZ [mm]');
% title('\DeltaZ [mm]');
% xlim([2 nmax])
% 
% % alpha
% figure
% plot(nvet,rad2deg(alpha_serie(2:nmax)),'c--o','Linewidth',2);
% xlabel('n');
% ylabel('\alpha [deg]');
% title('\alpha [deg]');
% xlim([2 nmax])

toc